@@ -12,10 +12,6 @@
     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
     See the License for the specific language governing permissions and
     limitations under the License.
-
-
-
-
 */
 
 #ifndef __TBB__concurrent_queue_impl_H
@@ -647,7 +643,7 @@ bool concurrent_queue_iterator_rep<T>::get_item( T*& item, size_t k ) {
 //! Constness-independent portion of concurrent_queue_iterator.
 /** @ingroup containers */
 template<typename Value>
-class concurrent_queue_iterator_base_v3 : no_assign {
+class concurrent_queue_iterator_base_v3 {
     //! Represents concurrent_queue over which we are iterating.
     /** NULL if one past last element in queue. */
     concurrent_queue_iterator_rep<Value>* my_rep;
@@ -670,10 +666,15 @@ protected:
 
     //! Copy constructor
     concurrent_queue_iterator_base_v3( const concurrent_queue_iterator_base_v3& i )
-    : no_assign(), my_rep(NULL), my_item(NULL) {
+    : my_rep(NULL), my_item(NULL) {
         assign(i);
     }
 
+    concurrent_queue_iterator_base_v3& operator=( const concurrent_queue_iterator_base_v3& i ) {
+        assign(i);
+        return *this;
+    }
+
     //! Construct iterator pointing to head of queue.
     concurrent_queue_iterator_base_v3( const concurrent_queue_base_v3<Value>& queue ) ;
 
@@ -768,8 +769,8 @@ public:
     {}
 
     //! Iterator assignment
-    concurrent_queue_iterator& operator=( const concurrent_queue_iterator& other ) {
-        this->assign(other);
+    concurrent_queue_iterator& operator=( const concurrent_queue_iterator<Container,typename Container::value_type>& other ) {
+        concurrent_queue_iterator_base_v3<typename tbb_remove_cv<Value>::type>::operator=(other);
         return *this;
     }
 
@@ -889,7 +890,7 @@ protected:
     //! Get size of queue
     ptrdiff_t __TBB_EXPORTED_METHOD internal_size() const;
 
-    //! Check if the queue is emtpy
+    //! Check if the queue is empty
     bool __TBB_EXPORTED_METHOD internal_empty() const;
 
     //! Set the queue capacity
@@ -980,6 +981,11 @@ protected:
         assign(i);
     }
 
+    concurrent_queue_iterator_base_v3& operator=( const concurrent_queue_iterator_base_v3& i ) {
+        assign(i);
+        return *this;
+    }
+
     //! Obsolete entry point for constructing iterator pointing to head of queue.
     /** Does not work correctly for SSE types. */
     __TBB_EXPORTED_METHOD concurrent_queue_iterator_base_v3( const concurrent_queue_base_v3& queue );
@@ -1029,8 +1035,8 @@ public:
     {}
 
     //! Iterator assignment
-    concurrent_queue_iterator& operator=( const concurrent_queue_iterator& other ) {
-        assign(other);
+    concurrent_queue_iterator& operator=( const concurrent_queue_iterator<Container,typename Container::value_type>& other ) {
+        concurrent_queue_iterator_base_v3::operator=(other);
         return *this;
     }
 
